@@ -11,6 +11,7 @@ Released under MIT license.
 from configbase import ConfigBase, ConfigBaseError
 from metadata import FieldsMetadata
 from genshi import XML
+import inspect
 
 class FillerBase(ConfigBase):
     """
@@ -70,8 +71,8 @@ class TableFiller(FillerBase):
     +-----------------------------------+--------------------------------------------+------------------------------+
     | Name                              | Description                                | Default                      |
     +===================================+============================================+==============================+
-    | __actions__                       | Whether or not to include actions in the   | True                         |
-    |                                   | output.                                    |                              |
+    | __actions__                       | An overridable function to define how to   | a function that creates an   |
+    |                                   | display action links in the form.          | edit and delete link.        |
     +-----------------------------------+--------------------------------------------+------------------------------+
     | __metadata_type__                 | How should we get data from the provider.  | FieldsMetadata               |
     +-----------------------------------+--------------------------------------------+------------------------------+
@@ -117,8 +118,12 @@ class TableFiller(FillerBase):
     def _get_list_data_value(self, field, values):
         l = []
         for value in values:
-            name = self.__provider__.get_view_field_name(value.__class__, self.__possible_field_names__)
-            l.append(unicode(getattr(value, name)))
+            if not isinstance(value, basestring):
+                name = self.__provider__.get_view_field_name(value.__class__, self.__possible_field_names__)
+                l.append(unicode(getattr(value, name)))
+            else:
+                #this is needed for postgres to see array values
+                return values
         return ', '.join(l)
 
     def _get_relation_value(self, field, value):
@@ -136,7 +141,37 @@ class TableFiller(FillerBase):
         if not hasattr(self, '__count__'):
             raise ConfigBaseError('Count not yet set for filler.  try calling get_value() first.')
         return self.__count__
+    
+    def _do_get_fields(self):
+        fields = super(TableFiller, self)._do_get_fields()
+        if '__actions__' not in self.__omit_fields__ and '__actions__' not in fields:
+            fields.insert(0, '__actions__')
+        return fields
 
+
+    def __actions__(self, obj):
+        """Override this function to define how action links should be displayed for the given record."""
+        primary_fields = self.__provider__.get_primary_fields(self.__entity__)
+        pklist = '/'.join(map(lambda x: str(getattr(obj, x)), primary_fields))
+        value = '<div><div>&nbsp;<a href="'+pklist+'/edit" style="text-decoration:none">edit</a>'\
+              '</div><div>'\
+              '<form method="POST" action="'+pklist+'" class="button-to">'\
+            '<input type="hidden" name="_method" value="DELETE" />'\
+            '<input class="delete-button" onclick="return confirm(\'Are you sure?\');" value="delete" type="submit" '\
+            'style="background: transparent; float:left; border:0; color: #286571; display: inline; margin: 0; padding: 0;"/>'\
+        '</form>'\
+        '</div></div>'
+        return value
+
+    def _do_get_provider_count_and_objs(self, **kw):
+        limit = kw.get('limit', None)
+        offset = kw.get('offset', None)
+        order_by = kw.get('order_by', None)
+        desc = kw.get('desc', False)
+        count, objs = self.__provider__.query(self.__entity__, limit, offset, self.__limit_fields__, order_by, desc)
+        self.__count__ = count
+        return count, objs
+    
     def get_value(self, values=None, **kw):
         """
         Get the values to fill a form widget.
@@ -151,40 +186,30 @@ class TableFiller(FillerBase):
          desc
           order the columns in descending order
         """
-        limit = kw.get('limit', None)
-        offset = kw.get('offset', None)
-        order_by = kw.get('order_by', None)
-        desc = kw.get('desc', False)
-        count, objs = self.__provider__.query(self.__entity__, limit, offset, self.__limit_fields__, order_by, desc)
-        self.__count__ = count
-        primary_fields = self.__provider__.get_primary_fields(self.__entity__)
+        count, objs = self._do_get_provider_count_and_objs(**kw)
         rows = []
         for obj in objs:
             row = {}
             for field in self.__fields__:
-                value = getattr(obj, field)
-                if 'password' in field.lower():
-                    row[field] = '******'
-                    continue
-                if isinstance(value, list):
-                    value = self._get_list_data_value(field, value)
-                elif self.__provider__.is_relation(self.__entity__, field) and value is not None:
-                    value = self._get_relation_value(field, value)
-                elif self.__provider__.is_binary(self.__entity__, field) and value is not None:
-                    value = '<file>'
+                field_method = getattr(self, field, None)
+                if inspect.ismethod(field_method):
+                    argspec = inspect.getargspec(field_method)
+                    if argspec and (len(argspec[0])-2>=len(kw) or argspec[2]):
+                        value = getattr(self, field)(obj, **kw)
+                    else:
+                        value = getattr(self, field)(obj)
+                else:
+                    value = getattr(obj, field)
+                    if 'password' in field.lower():
+                        row[field] = '******'
+                        continue
+                    elif isinstance(value, list):
+                        value = self._get_list_data_value(field, value)
+                    elif self.__provider__.is_relation(self.__entity__, field) and value is not None:
+                        value = self._get_relation_value(field, value)
+                    elif self.__provider__.is_binary(self.__entity__, field) and value is not None:
+                        value = '<file>'
                 row[field] = unicode(value)
-            #xxx: make this overridable
-            if self.__actions__:
-                pklist = '/'.join(map(lambda x: str(row[x]), primary_fields))
-                value = '<div><div>&nbsp;<a href="'+pklist+'/edit" style="text-decoration:none">edit</a>'\
-                      '</div><div>'\
-                      '<form method="POST" action="'+pklist+'" class="button-to">'\
-                    '<input type="hidden" name="_method" value="DELETE" />'\
-                    '<input class="delete-button" onclick="return confirm(\'Are you sure?\');" value="delete" type="submit"'\
-                    'style="background: transparent; float:left; border:0; color: #286571; display: inline; margin: 0; padding: 0;"/>'\
-                '</form>'\
-                '</div></div>'
-                row['__actions__'] = value
             rows.append(row)
         return rows
 
