@@ -10,8 +10,9 @@ Released under MIT license.
 import inspect
 from tw.api import Widget
 from tw.forms import HiddenField, TableForm
-from viewbase import ViewBase
+from viewbase import ViewBase, ViewBaseError
 from formencode import Schema, All
+from formencode import Validator
 from sprox.validators import UniqueValue
 from formencode.validators import UnicodeString, String
 from widgetselector import SAWidgetSelector
@@ -24,6 +25,12 @@ class FilteringSchema(Schema):
     filter_extra_fields = True
     allow_extra_fields = True
 
+class Field(object):
+    """Used to handle the case where you want to override both a validator and a widget for a given field"""
+    def __init__(self, widget=None, validator=None):
+        self.widget = widget
+        self.validator = validator
+
 class FormBase(ViewBase):
     """
 
@@ -92,7 +99,6 @@ class FormBase(ViewBase):
     One of the more useful things sprox does for you is to fill in the arguments to a drop down automatically.
     Here is the userform, limited to just the town field, which gets populated with the towns.
 
-
     >>> from sprox.formbase import FormBase
     >>> class UserOnlyTownForm(FormBase):
     ...    __model__ = User
@@ -100,25 +106,31 @@ class FormBase(ViewBase):
     >>>
     >>> town_form = UserOnlyTownForm(session)
     >>>
-    >>> print town_form()
-    <form xmlns="http://www.w3.org/1999/xhtml" action="" method="post" class="required tableform">
+    >>> print town_form() # doctest: +XML
+    <form action="" method="post" class="required tableform">
         <div>
-                <input type="hidden" name="sprox_id" class="hiddenfield" id="sprox_id" value="" />
+                <input type="hidden" id="sprox_id" class="hiddenfield" name="sprox_id" value="" />
         </div>
-        <table border="0" cellspacing="0" cellpadding="2">
-            <tr id="town.container" class="even" title="">
+        <table border="0" cellspacing="0" cellpadding="2" >
+            <tr class="even" id="town.container" title="" >
                 <td class="labelcol">
                     <label id="town.label" for="town" class="fieldlabel">Town</label>
                 </td>
-                <td class="fieldcol">
+                <td class="fieldcol" >
                     <select name="town" class="propertysingleselectfield" id="town">
-            <option value="1">Arvada</option><option value="2">Denver</option><option value="3">Golden</option><option value="4">Boulder</option><option value="" selected="selected">-----------</option>
+            <option value="1">Arvada</option>
+            <option value="2">Denver</option>
+            <option value="3">Golden</option>
+            <option value="4">Boulder</option>
+            <option value="" selected="selected">-----------</option>
     </select>
                 </td>
-            </tr><tr id="submit.container" class="odd" title="">
+            </tr>
+            <tr class="odd" id="submit.container" title="" >
                 <td class="labelcol">
+                    <label id="submit.label" for="submit" class="fieldlabel"></label>
                 </td>
-                <td class="fieldcol">
+                <td class="fieldcol" >
                     <input type="submit" class="submitbutton" value="Submit" />
                 </td>
             </tr>
@@ -126,11 +138,15 @@ class FormBase(ViewBase):
     </form>
 
     Forms created with sprox can be validated as you would any other widget.
-
-    >>> town_form.validate(params={'town':1})
+    >>> class UserOnlyTownForm(FormBase):
+    ...    __model__ = User
+    ...    __limit_fields__ = ['town']
+    ...    __required_fields__ = ['town']
+    >>> town_form = UserOnlyTownForm(session)
+    >>> town_form.validate(params={'sprox_id':1})
     Traceback (most recent call last):
     ...
-    Invalid: sprox_id: Missing value
+    Invalid: town: Missing value
 
 
 
@@ -164,10 +180,44 @@ class FormBase(ViewBase):
             self.__validator_selector__ = self.__validator_selector_type__(self.__provider__)
         if self.__field_validator_types__ is None:
             self.__field_validator_types__ = {}
+        if self.__dropdown_field_names__ is None:
+            self.__dropdown_field_names__ = ['name', '_name', 'description', '_description']
 
-    def validate(self, params):
+        #bring in custom declared validators 
+        for attr in dir(self):
+            if not attr.startswith('__'):
+                value = getattr(self, attr)
+                if isinstance(value, Field):
+                    widget = value.widget
+                    if isinstance(widget, Widget):
+                        if not getattr(widget, 'id', None):
+                            raise ViewBaseError('Widgets must provide an id argument for use as a field within a ViewBase')
+                        self.__add_fields__[attr] = widget
+                    try:
+                        if issubclass(widget, Widget):
+                            self.__field_widget_types__[attr] = widget
+                    except TypeError:
+                        pass
+                    validator = value.validator
+                    if isinstance(validator, Validator):
+                        self.__field_validators__[attr] = validator
+                    try:
+                        if issubclass(validator, Validator):
+                            self.__field_validator_types__[attr] = validator
+                    except TypeError:
+                        pass
+                if isinstance(value, Validator):
+                    self.__field_validators__[attr] = value
+                    continue
+                try:
+                    if issubclass(value, Validator):
+                        self.__field_validator_types__[attr] = value
+                except TypeError:
+                    pass
+    
+    def validate(self, params, state=None):
         """A pass-thru to the widget's validate function."""
-        return self.__widget__.validate(params)
+        return self.__widget__.validate(params, state)
 
     def _do_get_widget_args(self):
         """Override this method to define how the class get's the
@@ -298,58 +348,65 @@ class AddRecordForm(FormBase):
     ...     display_name           = TextField
     ...     verify_password        = PasswordField('verify_password')
     >>> registration_form = RegistrationForm()
-    >>> print registration_form()
-    <form xmlns="http://www.w3.org/1999/xhtml" action="" method="post" class="required tableform">
+    >>> print registration_form() # doctest: +XML
+    <form action="" method="post" class="required tableform">
         <div>
-                <input type="hidden" name="sprox_id" class="hiddenfield" id="sprox_id" value="" />
+                <input type="hidden" id="sprox_id" class="hiddenfield" name="sprox_id" value="" />
         </div>
-        <table border="0" cellspacing="0" cellpadding="2">
-            <tr id="user_name.container" class="even" title="">
+        <table border="0" cellspacing="0" cellpadding="2" >
+            <tr class="even" id="user_name.container" title="" >
                 <td class="labelcol">
                     <label id="user_name.label" for="user_name" class="fieldlabel required">User Name</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="text" name="user_name" class="textfield required" id="user_name" value="" />
+                <td class="fieldcol" >
+                    <input type="text" id="user_name" class="textfield required" name="user_name" value="" />
                 </td>
-            </tr><tr id="email_address.container" class="odd" title="">
+            </tr>
+            <tr class="odd" id="email_address.container" title="" >
                 <td class="labelcol">
                     <label id="email_address.label" for="email_address" class="fieldlabel required">Email Address</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="text" name="email_address" class="textfield required" id="email_address" value="" />
+                <td class="fieldcol" >
+                    <input type="text" id="email_address" class="textfield required" name="email_address" value="" />
                 </td>
-            </tr><tr id="display_name.container" class="even" title="">
+            </tr>
+            <tr class="even" id="display_name.container" title="" >
                 <td class="labelcol">
                     <label id="display_name.label" for="display_name" class="fieldlabel">Display Name</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="text" name="display_name" class="textfield" id="display_name" value="" />
+                <td class="fieldcol" >
+                    <input type="text" id="display_name" class="textfield" name="display_name" value="" />
                 </td>
-            </tr><tr id="password.container" class="odd" title="">
+            </tr>
+            <tr class="odd" id="password.container" title="" >
                 <td class="labelcol">
                     <label id="password.label" for="password" class="fieldlabel required">Password</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="password" name="password" class="required passwordfield" id="password" value="" />
+                <td class="fieldcol" >
+                    <input type="password" id="password" class="required passwordfield" name="password" value="" />
                 </td>
-            </tr><tr id="verify_password.container" class="even" title="">
+            </tr>
+            <tr class="even" id="verify_password.container" title="" >
                 <td class="labelcol">
                     <label id="verify_password.label" for="verify_password" class="fieldlabel">Verify Password</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="password" name="verify_password" class="passwordfield" id="verify_password" value="" />
+                <td class="fieldcol" >
+                    <input type="password" id="verify_password" class="passwordfield" name="verify_password" value="" />
                 </td>
-            </tr><tr id="town_id.container" class="odd" title="">
+            </tr>
+            <tr class="odd" id="town_id.container" title="" >
                 <td class="labelcol">
                     <label id="town_id.label" for="town_id" class="fieldlabel">Town Id</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="text" name="town_id" class="textfield" id="town_id" value="" />
+                <td class="fieldcol" >
+                    <input type="text" id="town_id" class="textfield" name="town_id" value="" />
                 </td>
-            </tr><tr id="submit.container" class="even" title="">
+            </tr>
+            <tr class="even" id="submit.container" title="" >
                 <td class="labelcol">
+                    <label id="submit.label" for="submit" class="fieldlabel"></label>
                 </td>
-                <td class="fieldcol">
+                <td class="fieldcol" >
                     <input type="submit" class="submitbutton" value="Submit" />
                 </td>
             </tr>
@@ -398,38 +455,41 @@ class DisabledForm(FormBase):
     ...     __model__ = User
     ...     __limit_fields__ = ['user_name', 'email_address']
     >>> disabled_user_form = DisabledUserForm()
-    >>> print disabled_user_form(values=dict(user_name='percious', email='chris@percious.com'))
-    <form xmlns="http://www.w3.org/1999/xhtml" action="" method="post" class="required tableform">
+    >>> print disabled_user_form(values=dict(user_name='percious', email='chris@percious.com'))  # doctest: +XML
+    <form action="" method="post" class="required tableform">
         <div>
-                <input type="hidden" name="user_name" class="hiddenfield" id="user_name" value="" />
-                <input type="hidden" name="email_address" class="hiddenfield" id="email_address" value="" />
-                <input type="hidden" name="sprox_id" class="hiddenfield" id="sprox_id" value="" />
+                <input type="hidden" id="user_name" class="hiddenfield" name="user_name" value="" />
+                <input type="hidden" id="email_address" class="hiddenfield" name="email_address" value="" />
+                <input type="hidden" id="sprox_id" class="hiddenfield" name="sprox_id" value="" />
         </div>
-        <table border="0" cellspacing="0" cellpadding="2">
-            <tr id="user_name.container" class="even" title="">
+        <table border="0" cellspacing="0" cellpadding="2" >
+            <tr class="even" id="user_name.container" title="" >
                 <td class="labelcol">
                     <label id="user_name.label" for="user_name" class="fieldlabel">User Name</label>
                 </td>
-                <td class="fieldcol">
-                    <input type="text" name="user_name" class="textfield" id="user_name" value="" disabled="disabled" />
+                <td class="fieldcol" >
+                    <input type="text" id="user_name" class="textfield" name="user_name" value="" disabled="disabled" />
                 </td>
-            </tr><tr id="email_address.container" class="odd" title="">
+            </tr>
+            <tr class="odd" id="email_address.container" title="" >
                 <td class="labelcol">
                     <label id="email_address.label" for="email_address" class="fieldlabel">Email Address</label>
                 </td>
-                <td class="fieldcol">
+                <td class="fieldcol" >
                     <textarea id="email_address" name="email_address" class="textarea" disabled="disabled" rows="7" cols="50"></textarea>
                 </td>
-            </tr><tr id="submit.container" class="even" title="">
+            </tr>
+            <tr class="even" id="submit.container" title="" >
                 <td class="labelcol">
+                    <label id="submit.label" for="submit" class="fieldlabel"></label>
                 </td>
-                <td class="fieldcol">
+                <td class="fieldcol" >
                     <input type="submit" class="submitbutton" value="Submit" />
                 </td>
             </tr>
         </table>
     </form>
-
+    
     You may notice in the above example that disabled fields pass in a hidden value for each disabled field.
 
     """
