@@ -108,6 +108,10 @@ class SAORMProvider(IProvider):
         field = self.get_field(entity, name)
         if isinstance(field, PropertyLoader):
             field = field.local_side[0]
+        # I am unsure what this is needed for, so it will be removed in the next version, and is for now
+        # commented until someone reports a bug.
+        #if not hasattr(field, 'type'):
+        #    return False
         return isinstance(field.type, Binary)
 
     def is_nullable(self, entity, name):
@@ -197,7 +201,7 @@ class SAORMProvider(IProvider):
         except NoResultFound:
             return True
         return False
-
+    
     def get_synonyms(self, entity):
         mapper = class_mapper(entity)
         return [prop.key for prop in mapper.iterate_properties if isinstance(prop, SynonymProperty)]
@@ -320,13 +324,13 @@ class SAORMProvider(IProvider):
         mapper = class_mapper(entity)
         relations = self.get_relations(entity)
         for relation in relations:
+            print relation
             #clear out those items which are not found in the params list.
             if relation not in params or not params[relation]:
                 related_items = getattr(obj, relation)
                 if related_items is not None:
                     if hasattr(related_items, '__iter__'):
-                        for item in getattr(obj, relation):
-                            getattr(obj, relation).remove(item)
+                        setattr(obj, relation, [])
                     else:
                         setattr(obj, relation, None)
                         
