@@ -1,13 +1,16 @@
-from sprox.formbase import FormBase, AddRecordForm, DisabledForm, EditableForm
-from sprox.test.base import setup_database, sorted_user_columns, SproxTest, setup_records, Example, Document
+from sprox.formbase import FormBase, AddRecordForm, DisabledForm, EditableForm, Field
+from sprox.viewbase import ViewBaseError
+from sprox.test.base import setup_database, sorted_user_columns, SproxTest, setup_records, Example, Document, assert_in_xml
 from sprox.test.model import User, Group
 from sprox.widgetselector import SAWidgetSelector
 from sprox.metadata import FieldsMetadata
 from nose.tools import raises, eq_
 from formencode import Invalid, Schema
-from formencode.validators import FieldsMatch
+from formencode.validators import FieldsMatch, NotEmpty, OpenId
 from tw.forms import PasswordField, TextField
 
+class MyTextField(TextField):pass
+
 session = None
 engine  = None
 connection = None
@@ -19,6 +22,15 @@ def setup():
 
 class UserForm(FormBase):
     __entity__ = User
+    
+class TestField:
+    
+    def setup(self):
+        self.field = Field(TextField, NotEmpty)
+        
+    def test_create(self):
+        assert self.field.widget == TextField
+        assert self.field.validator == NotEmpty
 
 class TestsEmptyDropdownWorks:
     def setup(self):
@@ -26,8 +38,10 @@ class TestsEmptyDropdownWorks:
         
     def test__widget__(self):
         rendered = self.base.__widget__()
-        assert """<td class="fieldcol">
-                <input type="submit" class="submitbutton" value="Submit" />
+        assert """<td class="fieldcol" >
+                <select name="town" class="propertysingleselectfield" id="town">
+        <option value="" selected="selected">-----------</option>
+</select>
             </td>""" in rendered, rendered
     
 
@@ -40,28 +54,122 @@ class TestFormBase(SproxTest):
     def test_create(self):
         pass
 
+    def test_formbase_with_validator_class(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = OpenId
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        try:
+            widget.validate({'user_name':'something'})
+        except Invalid, e:
+            assert u'"something" is not a valid OpenId (it is neither an URL nor an XRI)' in e.msg, e.msg
+
+    def test_formbase_with_validator_instance(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = OpenId()
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        try:
+            widget.validate({'user_name':'something'})
+        except Invalid, e:
+            assert u'"something" is not a valid OpenId (it is neither an URL nor an XRI)' in e.msg, e.msg
+
+    def test_formbase_with_field_validator_instance(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = Field(validator=OpenId())
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        try:
+            widget.validate({'user_name':'something'})
+        except Invalid, e:
+            assert u'"something" is not a valid OpenId (it is neither an URL nor an XRI)' in e.msg, e.msg
+
+    def test_formbase_with_field_validator_class(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = Field(validator=OpenId())
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        try:
+            widget.validate({'user_name':'something'})
+        except Invalid, e:
+            assert u'"something" is not a valid OpenId (it is neither an URL nor an XRI)' in e.msg, e.msg
+
+    def test_formbase_with_field_widget_class(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = Field(MyTextField)
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        assert isinstance(widget.children['user_name'], MyTextField)
+
+    def test_formbase_with_field_widget_instance(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = Field(MyTextField('user_name'))
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        assert isinstance(widget.children['user_name'], MyTextField)
+
+    @raises(ViewBaseError)
+    def test_formbase_with_field_widget_instance_no_id(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = Field(MyTextField())
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        assert isinstance(widget.children['user_name'], MyTextField)
+
+
+    def test_formbase_with_field_widget_and_validator_instance(self):
+        class UserForm(FormBase):
+            __entity__ = User
+            user_name = Field(MyTextField, OpenId)
+        user_form = UserForm(session)
+        widget = user_form.__widget__
+        assert isinstance(widget.children['user_name'], MyTextField)
+        try:
+            widget.validate({'user_name':'something'})
+        except Invalid, e:
+            assert u'"something" is not a valid OpenId (it is neither an URL nor an XRI)' in e.msg, e.msg
+
     def test__fields__(self):
         eq_(sorted_user_columns, sorted(self.base.__fields__))
 
     def test__widget__(self):
         rendered = self.base.__widget__()
-        assert """<td class="fieldcol">
+        assert_in_xml("""<tr class="even" id="submit.container" title="" >
+            <td class="labelcol">
+                <label id="submit.label" for="submit" class="fieldlabel"></label>
+            </td>
+            <td class="fieldcol" >
                 <input type="submit" class="submitbutton" value="Submit" />
-            </td>""" in rendered, rendered
+            </td>
+        </tr>""", rendered)
 
+    @raises(ViewBaseError)
+    def test_form_field_with_no_id(self):
+        class BogusUserForm(FormBase):
+            __entity__ = User
+            field = TextField()
+        bogus = BogusUserForm(session)
+        
     def test_entity_with_synonym(self):
         class DocumentForm(FormBase):
             __entity__ = Document
         form = DocumentForm(session)
         rendered = form()
-        assert """<tr id="address.container" class="odd" title="">
+        assert_in_xml("""<tr class="odd" id="address.container" title="" >
             <td class="labelcol">
                 <label id="address.label" for="address" class="fieldlabel">Address</label>
             </td>
-            <td class="fieldcol">
-                <input type="text" name="address" class="textfield" id="address" value="" />
+            <td class="fieldcol" >
+                <input type="text" id="address" class="textfield" name="address" value="" />
             </td>
-        </tr>""" in rendered, rendered
+        </tr>""", rendered)
 
     def test_entity_with_dropdown_field_names(self):
         class UserFormFieldNames(FormBase):
@@ -69,9 +177,15 @@ class TestFormBase(SproxTest):
             __dropdown_field_names__ = ['group_name']
         form = UserFormFieldNames(session)
         rendered = form()
-        assert """<select name="groups" class="propertymultipleselectfield" id="groups" multiple="multiple" size="5">
-        <option value="1">0</option><option value="2">1</option><option value="3">2</option><option value="4">3</option><option value="5">4</option>
-</select>""" in rendered, rendered
+        assert_in_xml("""<td class="fieldcol" >
+                <select name="groups" class="propertymultipleselectfield" id="groups" multiple="multiple" size="5">
+        <option value="1">0</option>
+        <option value="2">1</option>
+        <option value="3">2</option>
+        <option value="4">3</option>
+        <option value="5">4</option>
+</select>
+            </td>""", rendered)
         
     def test_entity_with_dropdown_field_names_dict(self):
         class UserFormFieldNames(FormBase):
@@ -79,9 +193,15 @@ class TestFormBase(SproxTest):
             __dropdown_field_names__ = {'groups':['group_name']}
         form = UserFormFieldNames(session)
         rendered = form()
-        assert """<select name="groups" class="propertymultipleselectfield" id="groups" multiple="multiple" size="5">
-        <option value="1">0</option><option value="2">1</option><option value="3">2</option><option value="4">3</option><option value="5">4</option>
-</select>""" in rendered, rendered
+        assert_in_xml( """<td class="fieldcol" >
+                <select name="groups" class="propertymultipleselectfield" id="groups" multiple="multiple" size="5">
+        <option value="1">0</option>
+        <option value="2">1</option>
+        <option value="3">2</option>
+        <option value="4">3</option>
+        <option value="5">4</option>
+</select>
+            </td>""", rendered)
         
         
     def test_require_field(self):
@@ -148,15 +268,14 @@ class TestEditableForm(SproxTest):
 
     def test__widget__(self):
         rendered = self.base.__widget__()
-        assert """<tr id="user_name.container" class="even" title="">
+        assert_in_xml("""<tr class="even" id="user_name.container" title="" >
             <td class="labelcol">
                 <label id="user_name.label" for="user_name" class="fieldlabel">User Name</label>
             </td>
-            <td class="fieldcol">
-                <input type="text" name="user_name" class="textfield has_error" id="user_name" value="asdf" />
-                <span class="fielderror">That value already exists</span>
+            <td class="fieldcol" >
+                <input type="text" id="user_name" class="textfield" name="user_name" value="" />
             </td>
-        </tr>""" in rendered, rendered
+        </tr>""", rendered)
         
 class TestDisabledForm(SproxTest):
     def setup(self):
@@ -171,7 +290,11 @@ class TestDisabledForm(SproxTest):
 
     def test__widget__(self):
         rendered = self.base.__widget__()
-        assert """<td class="fieldcol">
-                <input type="text" name="user_name" class="textfield has_error" id="user_name" value="asdf" disabled="disabled" />
-                <span class="fielderror">That value already exists</span>
-            </td>""" in rendered, rendered
\ No newline at end of file
+        assert_in_xml( """<tr class="even" id="user_name.container" title="" >
+            <td class="labelcol">
+                <label id="user_name.label" for="user_name" class="fieldlabel">User Name</label>
+            </td>
+            <td class="fieldcol" >
+                <input type="text" id="user_name" class="textfield" name="user_name" value="" disabled="disabled" />
+            </td>
+        </tr>""", rendered)
\ No newline at end of file
